# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"logger": "10_utilities.ipynb",
         "GPSTime": "00_core.ipynb",
         "LeapSeconds": "01_leapseconds.ipynb",
         "gps2utc": "01_leapseconds.ipynb",
         "utc2gps": "01_leapseconds.ipynb",
         "arange_gpstime": "10_utilities.ipynb",
         "validate_gps_week": "10_utilities.ipynb",
         "display_distro_statement": "11_logutils.ipynb",
         "Colors": "11_logutils.ipynb",
         "ThemeField": "11_logutils.ipynb",
         "color_text": "11_logutils.ipynb",
         "BasicTheme": "11_logutils.ipynb",
         "BasicColorTheme": "11_logutils.ipynb",
         "AlignedColorFormatter": "11_logutils.ipynb",
         "correct_week": "20_datetime.ipynb",
         "arange_datetime": "20_datetime.ipynb",
         "tow2datetime": "20_datetime.ipynb",
         "datetime2tow": "20_datetime.ipynb",
         "tow2zcount": "20_datetime.ipynb",
         "zcount2tow": "20_datetime.ipynb",
         "datetime2zcount": "20_datetime.ipynb",
         "zcount2datetime": "20_datetime.ipynb"}

modules = ["core.py",
           "leapseconds.py",
           "utilities.py",
           "logutils.py",
           "datetime.py"]

doc_url = "https://the-aerospace-corporation.github.io/gps_time/"

git_url = "https://github.com/the-aerospace-corporation/gps_time/tree/master/"

def custom_doc_links(name):
    return None
